#!/ffp/bin/sh

# PROVIDE: minidlna

########################################################################################################################
#MiniDLNA (aka ReadyDLNA) is server software with the aim of being fully compliant with DLNA/UPnP-AV clients.          #
#MiniDLNA dependencies required:ffmpeg with it's own dependencies, libjpeg, flac, libexif, libid3tag, libogg, libvorbis#
#sqlite, gettext, intltool                                                                                             #
#                                                                                                                      #
#http://sourceforge.net/projects/minidlna/                                                                             #
#                                                                                                                      #
#MiniDLNA daemon startup-shutdown-control script for FFP by barmalej2                                                  #
########################################################################################################################
# Logging is enabled by default. Logs are located in /ffp/var/log/minidlna.log file.
# Verbosity  of logs is controlled in minidlna config_file-/ffp/etc/minidlna.conf by parameter-log_level.
# Default action is adjusted to log errors only.

. /ffp/etc/ffp.subr
# Make sure the configuration file exists. If yes, import settings.
[ -f /ffp/etc/minidlna.conf ] || { echo -e "\e[1;31;20mERROR WARNING: MiniDLNA config file /ffp/etc/minidlna.conf was not found. Exiting...\e[0m"; exit 1; } && . /ffp/etc/minidlna.conf

name="minidlna"
command="/ffp/bin/$name"
home_dir="$db_dir"
pid_dir="/ffp/var/run/$name/"
config_file="/ffp/etc/$name.conf"
required_files="$command"
runas_user="$user"
minidlna_args="-f $config_file"	#Minidlna daemon run args. Important-stop minidlna daemon service first, before making changes in this row! 
if	[ -n "$minidlna_args" ]
then
	command="$command $minidlna_args"
fi

start_cmd="minidlna_start"
stop_cmd="minidlna_stop"
status_cmd="minidlna_status"
extra_commands="reload rescan"
reload_cmd="minidlna_reload"
rescan_cmd="minidlna_rescan"

add_user()
{
	/ffp/bin/id -g "$runas_user" &>/dev/null || /ffp/sbin/groupadd -r "$runas_user" >/dev/null
	/ffp/bin/id -u "$runas_user" &>/dev/null || /ffp/sbin/useradd -r -g "$runas_user" -d "$home_dir" -s /ffp/bin/false -c MiniDLNA "$runas_user" >/dev/null
}

check_home()
{
	# Create dirs if necessary 
	# If $home_dir or $pid_dir was owned by the different user from current, we need to own them
	for dir in $home_dir $pid_dir; do
		if	[ ! -d "$dir" ]
		then
			mkdir -p "$dir" || { echo -e "\e[1;31;20mFail!!! ERROR WARNING: Can't create $dir directory-aborting start of $name due to this error!!!\e[0m"; exit 1; }
			chown -R "$runas_user":"$runas_user" "$dir"
		else
			if	[ "$(stat -c '%U' "$dir")" != "$runas_user" ]
			then
				chown -R "$runas_user":"$runas_user" "$dir"
			fi
		fi
	done
	# Increase max_user_watches
	echo "66000" > /proc/sys/fs/inotify/max_user_watches
}

check_status()
{
	status=$(pgrep -f "$command")
	retvalue=$?
}

minidlna_start()
{
	check_status
	if	[ "$retvalue" = "1" ]; then
		add_user
		check_home
		echo -n "Starting $name..."
		$command
		sleep 1
		check_status
		if	[ "$retvalue" = "0" ]; then
			echo -e "done! \e[1;32;20m[ OK ]\e[0m"
		else
			echo -e "\e[1;31;20m[ Fail!!! ] ERROR WARNING: $name can not start!!!\e[0m"
			exit 1
		fi
	else
		if	[ "$retvalue" = "0" ]; then
			echo -e "PROCESS STATUS INFO: \e[1;33;20m$name is already running with process ID(s):"${status[*]}"\e[0m"
			ps -fj -C $name
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-start command failed!!!\e[0m"
			exit 1
		fi
	fi
}

minidlna_stop()
{
	check_status
	if	[ "$retvalue" = "0" ]; then
		echo -n "Stopping $name..."
		pkill -fP1 "$command"
		local timeout=0
		while [ $timeout -lt 5 ]; do
			sleep 1
			check_status
			if	[ "$retvalue" = "1" ]; then
				echo -e "done! \e[1;32;20m[ OK ]\e[0m"
				return 0
			fi
			let timeout=timeout+1
		done
		if	[ "$retvalue" = "0" ]; then
			echo -e "\e[1;31;20m[ Fail!!! ] ERROR WARNING: $name failed to exit cleanly within $timeout seconds. Timeout reached!\e[0m"
			echo -n "Stopping $name by force..."
			pkill -9 -f "$command"
			local timeout=0
			while [ $timeout -lt 2 ]; do
				check_status
				if	[ "$retvalue" = "1" ]; then
					echo -e "done! \e[1;32;20m[ OK ]\e[0m"
					return 0
				fi
				sleep 1
				let timeout=timeout+1
			done
			echo -e "\e[1;31;20m[ Fail!!! ] ERROR WARNING: $name can not be stopped!!!\e[0m"
			exit 1
		else
			echo -e "\e[1;31;20m[ Fail!!! ]ERROR WARNING: Can not determine running status of $name-stopping command failed!!!\e[0m"
			exit 1
		fi
	else
		if	[ "$retvalue" = "1" ]; then
			echo -e "PROCESS STATUS INFO: \e[1;33;20m$name is not running\e[0m"
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-stopping command failed!!!\e[0m"
			exit 1
		fi
	fi
}

minidlna_status()
{
	check_status
	echo -n "PROCESS STATUS INFO: "
	if	[ "$retvalue" = "0" ]
	then
		echo -e "\e[1;33;20m$name is running with process ID(s):"${status[*]}"\e[0m"
		ps -fj -C $name
	else
		if	[ "$retvalue" = "1" ]
		then
			echo -e "\e[1;33;20m$name is not running\e[0m"
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-status command failed!!!\e[0m"
			exit 1
		fi
	fi
}

minidlna_reload()
{
	# This command will reload minidlna config file
	check_status
	if	[ "$retvalue" = "0" ]
	then
		echo -n "Reloading $name config file: $config_file..."
		pkill -HUP -fP1 "$command"
		if	[ "$?" = "0" ]
		then
			echo -e "done! \e[1;32;20m[ OK ]\e[0m"
		else
			echo -e "\e[1;31;20mERROR WARNING:[ FAIL ]!!!\e[0m"
		fi
	else
		if	[ "$retvalue" = "1" ]
		then
			echo -e "PROCESS STATUS INFO: \e[1;33;20m$name is not running\e[0m"
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-reload command failed!!!\e[0m"
			exit 1
		fi
	fi
}

minidlna_rescan()
{
	check_status
	if	[ "$retvalue" = "1" ]
	then
		minidlna_args="$minidlna_args -R"
		command="/ffp/bin/$name $minidlna_args"
		minidlna_start && { echo -e "\e[1;33;20mFull MiniDLNA database rescan is in progress...\e[0m"; echo -e "\e[1;33;20mThis process can take for a while, depending on size and number of files used in media library!\e[0m"; }
	else
		if	[ "$retvalue" = "0" ]
		then
			minidlna_stop
			minidlna_args="$minidlna_args -R"
			command="/ffp/bin/$name $minidlna_args"
			minidlna_start && { echo -e "\e[1;33;20mFull MiniDLNA database rescan is in progress...\e[0m"; echo -e "\e[1;33;20mThis process can take for a while, depending on size and number of files used in media library!\e[0m"; }
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-rescan command failed!!!\e[0m"
			exit 1
		fi
	fi
}

run_rc_command "$1"

